function [b,v,q] = gmm(...
    b0,fmin,fu,nstep,...
    wtype,wlag,varargin)

% Adapted from Craig Burnside, January 2000
% Annotations refer to DeJong and Dave 2e, section 12.2
%   Inputs:
%       b0:     initial parameters
%       fmin:   orthogonality conditions
%       fu:     residuals
%       nstep:  number of times to update weighting matrix
%       For covariance matrix:
%           wtype:  choice of numerical approximation
%           wlag:   Number of lags used in window
%       (varargin) weighting matrix
%       	if empty, then W0 is identity matrix
%           if scalar, then W0 is based on b0
%           if matrix, then W0 is specified by user
%   Outputs:
%       b:      estimates
%       v:      asymptotic covariance matrix of estimator
%       q:      objective function value

% Dimensions
u0 = fu(b0);
[T,k] = size(u0);

% Initial weighting matrix
if isempty(varargin)
    W = eye(k);
elseif numel(varargin{1}) == 1
    W = inv(gmms0(u0,wtype,wlag));
else
    W = varargin{1};
end

% 1st Step of GMM
[b,q,converged] = quadmin(fmin,b0,W); % Eq. 12.3

if ~converged
    b = b * NaN;
    v = NaN(length(b), length(b));
    q = q * NaN;
    return;
end

u = fu(b);
s0 = gmms0(u,wtype,wlag);  % Eq. 12.4
W = inv(s0);

iter = 1;
while iter < nstep
    b0 = b;

    [b,q,converged] = quadmin(fmin,b0,W); % Eq. 12.3
    
    if ~converged
        b = b * NaN;
        v = NaN(length(b), length(b));
        q = q * NaN;
        return;
    end

    g0 = fmin(b);               % Eq. 12.2
    dg = dfeval(fmin,g0,b,0);   % Eq. 12.8
    v = inv(dg'*(s0\dg))/T;     % Eq. 12.9 divided by T
    q = T*q;

    iter = iter+1;
end