function [x1,f1,converged] = quadmin(func,x0,W)

% Adapted from Craig Burnside, January 2000
% Annotations refer to DeJong and Dave 2e, section 12.2
%   Inputs:
%       func:       orthogonality conditions
%       x0:         inital parameters
%       W:          Weighting matrix
%   Outputs:
%       x1:         parameters that minimize objective function
%       f1:         objective function value at minimum
%       niter:      number of iterations to converge

%   Convergence criterion
xtol = 1e-6;
%   Maximum iterations
maxiter = 400;
%   Step distance tolerance
sstol = 1e-12;

[k,~] = size(x0);

g0 = feval(func,x0);        % Eq. 12.2
dg0 = dfeval(func,g0,x0,0); % Eq. 12.8

f0 = g0'*W*g0;              % Eq. 12.3
df0 = g0'*W*dg0;

h0 = inv(dg0'*W*dg0);       % Eq. 12.9

xi = -h0*(df0');
convcrit = 0;
niter = 1;

while convcrit==0
    s = 1;
    while s > sstol
        dx = s*xi;
        x1 = x0+dx;
        g1 = feval(func,x1);% Eq. 12.2
        f1 = g1'*W*g1;      % Eq. 12.3
        if f1 < f0
            g0 = g1;
            f0 = f1;
            x0 = x1;
            s = 0;
        end
        s = s*0.5;
    end
    test_ = max(abs(xi)./max([abs(x0),ones(k,1)],[],2));
    if niter==maxiter
        convcrit = 1;
    elseif test_<xtol
        convcrit = 1;
    else
        dg0 = dfeval(func,g0,x0,0);	% Eq. 12.8
        df0 = g0'*W*dg0;
        h0 = inv(dg0'*W*dg0);       % Eq. 12.9
        xi = -h0*(df0');
    end
    niter = niter + 1;
end

if niter > maxiter
    converged = 0;
    disp('quadmin: maximum iterations reached')
else
    converged = 1;
end