% Equilibrium outcomes
% clear; close all; clc
figtype = 'M';
saving = 'on';
plotops
warning('off')

% Preferences: CRRA, EZ
prefs = 'EZ';

% Model options: AELR, DRT, AELR Intuition, DRT Intuition
modstr = {'AELR','AELR Intuition','DRT','DRT Intuition'};
nmod = numel(modstr);
savename = ['Results/asset_moments_',prefs];

% Alternative Calibrations
iess = [linspace(.01,.99,500),linspace(1.01,3,500)];
nies = numel(iess);

% Set random number seed
mtstream = RandStream('mt19937ar','seed',123456);
RandStream.setGlobalStream(mtstream);

% Load GMM moment estimates
load('GMM/gmm_data')
pwd0 = pwd;
%   Empirical targets
O.smmidx = [M.mu_rd,M.mu_rf];
%   GMM estimates
PsiD = PsiD(O.smmidx);
% OmegaD = OmegaD(O.smmidx,O.smmidx);
OmegaD = eye(2);

cd('Baseline')

% Load variable indexes
V = variables_cs;
Vlin = variables;

% Expected dividend, risk free return, and equity premium
Erds = NaN(nies,nmod);
Erfs = NaN(nies,nmod);
Eeps = NaN(nies,nmod);
Epds = NaN(nies,nmod);
kappad1s = NaN(nies,nmod);
Js = NaN(nies,nmod);

% Set parameters
P0.gamma = 10;
P0.beta = 0.9975;
P0.rhoa = 0;
P0.sigy = 0;
P0.muy = 0.0015;
P0.mud = 0.0015;
P0.psid = 0;
P0.pidy = 0;
    
for imod = 1:nmod
    % Set model
    O.model = modstr{imod};  
    disp(O.model); 
    P = P0;
    
    % Special cases
    if any(strcmp(O.model,{'AELR Intuition','DRT Intuition'}))
        P.muy = 0;
        P.mud = 0;
    end
    
    % Set preference dependent parameters
    if  any(strcmp(O.model,{'AELR','AELR Intuition'}))
        P.siga = 0.005;
        P.omega = 0;
    elseif any(strcmp(O.model,{'DRT','DRT Intuition'}))
        P.siga = 0.005;
        P.omega = P.beta;
    end

    % Solve model
    x0s = zeros(8,nies);
    Rflags = zeros(nies,1);
    disp(P)
    for iies = 1:nies
        P.psi = iess(iies);
        if strcmp(prefs,'CRRA')
            P.gamma = 1/P.psi;
        end
        P.theta = (1-P.gamma)/(1-1/P.psi);
        % Use linear solution as guess
        Slin = steadystate(P);
        [T,~,C,eu] = linmodel(P,Slin,Vlin);
        if ~all(eu == 1)
            disp('linear solution does no exist or is not unique')
        end
        x0 = [...
            Slin.kappay0,Slin.kappay1,C(Vlin.zy),T(Vlin.zy,Vlin.a),...
            Slin.kappad0,Slin.kappad1,C(Vlin.zd),T(Vlin.zd,Vlin.a)]';
        % Campbell-shiller solution
        [S,R,x] = solution_cs(P,x0);
        if R.flag
            Erds(iies,imod) = R.Erd;
            Erfs(iies,imod) = R.Erf;
            Eeps(iies,imod) = R.Eep;
            Epds(iies,imod) = R.Epd;
            kappad1s(iies,imod) = S.kappad1;
            x0s(:,iies) = x;
            Rflags(iies) = 1;
            Psidiff = [R.Erd,R.Erf]' - PsiD;
            Js(iies,imod) = (Psidiff'/OmegaD)*Psidiff;
        end
        if ~Rflags(iies)
            disp(['Solver did not converge with IES=' num2str(P.psi)])
        end
    end
end

% AELR spec, limit psi -> infty
O.model = modstr{1};
P = P0; 
% Set preference dependent parameters
if  any(strcmp(O.model,{'AELR','AELR Intuition'}))
    P.siga = 0.005;
    P.omega = 0;
elseif any(strcmp(O.model,{'DRT','DRT Intuition'}))
    P.siga = 0.02;
    P.omega = P.beta;
end
P.psi = 1e16;
if strcmp(prefs,'CRRA')
    P.gamma = 1/P.psi;
end
P.theta = (1-P.gamma)/(1-1/P.psi);
% Use linear solution as guess
Slin = steadystate(P);
[T,~,C,eu] = linmodel(P,Slin,Vlin);
if ~all(eu == 1)
    disp('linear solution does no exist or is not unique')
end
x0 = [...
    Slin.kappay0,Slin.kappay1,C(Vlin.zy),T(Vlin.zy,Vlin.a),...
    Slin.kappad0,Slin.kappad1,C(Vlin.zd),T(Vlin.zd,Vlin.a)]';
% Campbell-shiller solution
[S,R,x] = solution_cs(P,x0);
Eeplim = R.Eep;
        
cd(pwd0)
%% Plot figures

% Plot options
ifigtype=0;
% 	Manuscript version
ifigtype=ifigtype+1;
figpos{ifigtype} = [1 4.5 6.5 4.5];
pad(ifigtype).topmarg = .1;
pad(ifigtype).leftmarg = .06;
pad(ifigtype).vspace = .10;
pad(ifigtype).hspace = .06;
pad(ifigtype).rightmarg = .02;
pad(ifigtype).botmarg = .09;
fontsize(ifigtype) = 10;
legfontsize(ifigtype) = 10;
legadj(ifigtype) = .075;
legheight(ifigtype) = 0;
figtypes{ifigtype} = 'M';
% % 	Presentation version
% ifigtype=ifigtype+1;
% figpos{ifigtype} = [1 1 4.5 2.85];
% pad(ifigtype).topmarg = .10;
% pad(ifigtype).leftmarg = .06;
% pad(ifigtype).vspace = .175;
% pad(ifigtype).hspace = .13;
% pad(ifigtype).rightmarg = .03;
% pad(ifigtype).botmarg = .13;
% fontsize(ifigtype) = 8;
% legfontsize(ifigtype) = 6;
% legadj(ifigtype) = .04;
% legheight(ifigtype) = .05;
% figtypes{ifigtype} = 'P';

% Preference-specific options
if strcmp(prefs,'EZ')
    type_leg = {'-','--d','-o','--','--x'};
    type_line = {'-','--','-','--','--'};
    type_mark = {'.','d','o','.','x'};
    ylims{1} = [0,8];
    ylims{2} = [-1,1];
    ylims{3} = [.995,1];
    ylims{4} = [0,100];
    ysteps{1} = 2;
    ysteps{2} = .5;
    ysteps{3} = .001;
    ysteps{4} = 25;
elseif strcmp(prefs,'CRRA')
    type_leg = {'-','-d','--o','--'};
    type_line = {'-','-','--','--'};
    type_mark = {'.','d','o','.'};
    ylims{1} = [0,8];
    ylims{2} = [-.04,.04];
    ylims{3} = [.995,.999];
    ylims{4} = [0,100];
    ysteps{1} = 2;
    ysteps{2} = .02;
    ysteps{3} = .001;
    ysteps{4} = 50;
end

% Common options
markersize = 3;
markergrid = linspace(min(iess),max(iess),9);
colors = {[0 0 0],[1 0 0],[0 0 1],[.5 0 .5],[0 .5 .5]}; 
plotdim = [2,2];
titlestr = {'Mean Risk-Free Rate ($E[\hat{r}_f]$)',...
            'Mean Equity Premium ($E[ep]$)',...
            'Price-Dividend Loading ($\kappa_1$)',...
            'Model Fit'};
        
ifig = 1;
for ifigtype = 1:numel(figtypes)
    if strcmp(figtypes{ifigtype},'M')
        ifig=ifig+1; figure(ifig); clf
        set(gcf,'Position',figpos{ifigtype},...
            'PaperPosition',[0 0 figpos{ifigtype}(3:4)],...
            'PaperSize',figpos{ifigtype}(3:4));
    end
    plotme = cat(3,Erfs,Eeps,kappad1s,Js);
    p1 = zeros(1,nmod);
    for isubplot = 1:prod(plotdim)
        if strcmp(figtypes{ifigtype},'M')
            subplot(plotdim(1),plotdim(2),isubplot); grid on; hold on; box on;
            [icol,irow] = ind2sub(plotdim([2 1]),isubplot);
        elseif strcmp(figtypes{ifigtype},'P')
            ifig=ifig+1; figure(ifig); grid on; hold on; box on;
            set(gcf,'Position',figpos{ifigtype}+[5*(isubplot-1),0,0,0],...
                'PaperPosition',[0 0 figpos{ifigtype}(3:4)],...
                'PaperSize',figpos{ifigtype}(3:4));
            set(gca,'Position',[pad(ifigtype).leftmarg,pad(ifigtype).botmarg,...
                1-pad(ifigtype).leftmarg-pad(ifigtype).rightmarg,1-pad(ifigtype).botmarg-pad(ifigtype).topmarg])
            irow = 2;
        end
        for imod = 1:nmod
            temp = plotme(:,imod,isubplot);
            if any(strcmp(modstr{imod},{'AELR','AELR Intuition'})) && isubplot < 3
                idx = iess < 1;
                plot(iess(idx),temp(idx),type_line{imod},'color',colors{imod});
                idx = iess > 1;
                plot(iess(idx),temp(idx),type_line{imod},'color',colors{imod});
            else
                plot(iess,temp,type_line{imod},'color',colors{imod});
            end
            if ~strcmp(type_mark{imod},'.')
                plot(markergrid,interp1(iess,temp,markergrid,'pchip'),...
                    type_mark{imod},'color',colors{imod},'MarkerSize',markersize);  
            end
            p1(imod) = plot(iess(1),temp(1),type_leg{imod},'color',colors{imod},'MarkerSize',markersize);
            % Add lim psi -> infty
            if isubplot == 2 && imod == 1
                xlims = get(gca,'xlim');
                plot(xlims,Eeplim*ones(2,1),'k-.','linewidth',.5)
            end
        end

        set(gca,'ylim',ylims{isubplot},'fontsize',fontsize(ifigtype))
        set(gca,'xtick',0:0.5:max(iess),'xlim',[min(iess),max(iess)],'fontsize',fontsize(ifigtype))
        set(gca,'ytick',ylims{isubplot}(1):ysteps{isubplot}:ylims{isubplot}(2))
        if strcmp(figtypes{ifigtype},'M')
            title(titlestr{isubplot},'fontsize',fontsize(ifigtype))
        end
        if irow == plotdim(1)
            xlabel('IES ($\psi$)','fontsize',fontsize(ifigtype))
        end
        %   Add Legend
        if (nmod>1 && isubplot==1) || strcmp(figtypes{ifigtype},'P')
           legstr = {...
                ['C ($\mu_y = ' num2str(P0.muy) '$)'],...
                'C ($\mu_y = 0$)',...
                ['R ($\mu_y = ' num2str(P0.muy) '$)'],...
                'R ($\mu_y = 0$)'};
            legend(p1,legstr,...
                'Orientation','horizontal',...
                'Location','NorthEast',...
                'Position',[.25,1-pad(ifigtype).topmarg+legadj(ifigtype),0.5,legheight(ifigtype)],...
                'FontSize',legfontsize(ifigtype),'interpreter','latex');
        end
    end

    % Resize axes
    if strcmp(figtypes{ifigtype},'M')
        for isubplot = 1:prod(plotdim)
            [col,row] = ind2sub(plotdim([2 1]),isubplot);
            width = (1-pad(ifigtype).leftmarg-pad(ifigtype).hspace*(plotdim(2)-1)-pad(ifigtype).rightmarg)/plotdim(2);
            height = (1-pad(ifigtype).topmarg-pad(ifigtype).vspace*(plotdim(1)-1)-pad(ifigtype).botmarg)/plotdim(1);
            left = pad(ifigtype).leftmarg + (col-1)*(width+pad(ifigtype).hspace);
            bottom = 1-pad(ifigtype).topmarg-row*height-(row-1)*pad(ifigtype).vspace;
            subplot(plotdim(1),plotdim(2),isubplot)
            set(gca,'Position',[left bottom width height])
        end
    end

    % Add zoom to second subplot
    isubplot = 2;
    if strcmp(figtypes{ifigtype},'M')
        axes('Parent',gcf,'OuterPosition',[.675 .545 .325 .18]); box on; hold on; grid on;
    elseif strcmp(figtypes{ifigtype},'P')
        figure(3);
        axes('Parent',gcf,'OuterPosition',[.35 .15 .65 .35]); box on; hold on; grid on;
    end
    idxsub = iess <= .5;
    iesssub = iess(idxsub);
    markergridsub = linspace(min(iesssub),max(iesssub),5);
    for imod = [3,4]
        temp = plotme(idxsub,imod,isubplot);
        idx = iesssub < 1;
        plot(iesssub(idx),temp(idx),type_line{imod},'color',colors{imod},'linewidth',.5);
        if ~strcmp(type_mark{imod},'.')
            plot(markergridsub,interp1(iesssub,temp,markergridsub,'pchip'),...
                type_mark{imod},'color',colors{imod},'MarkerSize',markersize-1,'linewidth',.5);  
        end
    end
    set(gca,'xlim',[0,.5],'xtick',0:.1:.5,'ylim',[0.02,.031],'fontsize',fontsize(ifigtype)-3)

    % Save figure
    if strcmp(saving,'on')    
        if strcmp(figtypes{ifigtype},'M')
            saveas(gcf,[savename,'.pdf'])
        elseif strcmp(figtypes{ifigtype},'P')
            for ifig = 1+(1:prod(plotdim))
                savenametemp = [savename '_pres' num2str(ifig-1)];
                saveas(ifig,[savenametemp,'.pdf'])
            end
        end
    end
end